"""PDFMagic (OpenWhisk action): convert a JPEG image to PDF.

Built as an Apache OpenWhisk action (i.e., serverless cloud function).
The image is received as a base64-encoded string of the JPEG binary content, and
the converted result is sent back as a base64-encoded string of the PDF binary
content.

Requirements:
* native dependencies:
  * imagemagick (command `convert`)
* Python dependencies:
  * delegator (package delegate.py)
"""

from base64 import b64decode, b64encode

import delegator

# The input image is written to this file.
IN_FILENAME = 'image.jpg'
# The output PDF is written to this file.
OUT_FILENAME = 'image.pdf'


def main(args):
    """Entrypoint of the action.

    Expected arguments:
    * image: base64-encoded string of the input JPEG image binary content

    Return payload:
    * pdf: base64-encoded string of the output PDF document binary content

    Error return payload:
    * error: message describing the error
    """
    if 'image' not in args:
        return {
            'error': 'missing parameter: image',
        }

    # Decode and write the image to a file.
    with open(IN_FILENAME, 'wb') as in_file:
        in_file.write(b64decode(args['image']))

    # Call conversion command, and wait for completion.
    delegator.run(f'/usr/bin/convert {IN_FILENAME} {OUT_FILENAME}',
                                 block=True)

    # Read and encode the output from a file.
    with open(OUT_FILENAME, 'rb') as out_file:
        pdf = b64encode(out_file.read()).decode()

    return {
        'pdf': pdf,
    }
